
# q-expansion of newform 507.2.f.b, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 507
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 1, 0], 1], [[0, 0, 0, 1], 1], [[0, 1, -1, 0], 1]]

hecke_ring_character_values = [[170, [-1, 0, 0, 0]], [340, [0, 0, 1, 0]]]
aps_data = [[0, 0, 0, 0], [0, 1, -1, 1], [0, 0, 0, 0], [-2, 0, 2, -3], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [5, -2, 5, 0], [0, 0, 0, 0], [0, 0, 0, 0], [6, -5, 6, 0], [7, 0, -7, 4], [0, 0, 0, 0], [1, -1, 0, 1], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 5, -5, 5], [2, 7, 2, 0], [0, 0, 0, 0], [8, 0, -8, -1], [0, -7, 7, -7], [0, 0, 0, 0], [0, 0, 0, 0], [-8, 11, -8, 0], [0, 0, 0, 0], [9, -9, 0, 9], [0, 0, 0, 0], [-7, -5, -7, 0], [0, 0, 0, 0], [0, 0, -1, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-7, 0, 0, 0], [0, 0, 0, 0], [-5, 0, 5, -14], [-11, 0, 0, 0], [11, 0, -11, -3], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-4, 4, 0, -4], [0, 0, 0, 0], [7, 0, -7, -9], [0, 0, 0, 0], [0, 0, 17, 0], [0, 15, -15, 15], [-11, -6, -11, 0], [0, 0, 0, 0], [-5, 0, 5, 12], [0, 0, 0, 0], [0, 0, 0, 0], [-15, 0, 15, -16], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-19, 0, 19, -9], [-12, 12, 0, -12], [0, 0, 0, 0], [0, 0, 25, 0], [0, 0, 0, 0], [-18, 0, 18, -1], [0, 0, 0, 0], [0, 19, -19, 19], [0, 0, 0, 0], [-21, 11, -21, 0], [0, 0, -29, 0], [0, 0, 0, 0], [20, 0, -20, 17], [0, 0, 0, 0], [0, 0, 0, 0], [31, 0, 0, 0], [0, -21, 21, -21], [15, 7, 15, 0], [0, 0, 0, 0], [0, 0, 0, 0], [11, 0, -11, 23], [0, 0, 0, 0], [-8, -15, -8, 0], [0, 0, 0, 0], [-20, 21, -20, 0], [0, 0, 0, 0], [0, 0, -35, 0], [-23, 23, 0, -23], [0, 0, 0, 0], [0, 0, 0, 0], [-24, 7, -24, 0], [0, 0, 0, 0], [21, 0, -21, -1], [0, 0, 0, 0], [0, 0, 0, 0], [-21, -2, -21, 0], [0, 0, 0, 0], [-7, -18, -7, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-8, 0, 0, 0], [21, 0, -21, 25], [41, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, -16, 0], [19, 8, 19, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -24, 24, -24], [-20, 0, 0, 0], [28, -19, 28, 0], [0, 0, 0, 0], [-5, 0, 5, -27], [-15, 0, 15, -29], [0, 0, 0, 0], [18, 11, 18, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [20, 0, -20, -9], [29, -29, 0, 29], [0, 0, 0, 0], [0, 0, 0, 0], [-11, -19, -11, 0], [0, 0, 0, 0], [-28, 0, 28, -3], [0, 0, 0, 0], [0, 0, 49, 0], [19, -31, 19, 0], [7, 0, -7, 30], [0, 0, 0, 0], [-10, 10, 0, -10], [0, 28, -28, 28], [0, 0, 0, 0], [15, -32, 15, 0], [0, 0, 0, 0], [-2, 0, 2, -29], [0, 0, 0, 0], [0, 0, 0, 0], [6, -31, 6, 0], [0, 0, 0, 0], [14, -14, 0, 14], [0, 0, 0, 0], [0, 0, -53, 0], [0, 0, 0, 0], [-31, 0, 31, -27], [0, 0, 0, 0], [0, -33, 33, -33], [0, 0, 0, 0], [31, -28, 31, 0], [0, 0, 0, 0], [0, 0, -55, 0], [0, 0, 0, 0], [0, 0, -40, 0], [0, 0, 0, 0], [0, 18, -18, 18], [0, 0, 0, 0], [0, 32, -32, 32], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-7, 34, -7, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [44, 0, 0, 0], [-59, 0, 0, 0]]
